((c-mode . ((c-file-style . "ruby")
            (indent-tabs-mode . nil)
            (show-trailing-whitespace . t)))
 (ruby-mode . ((show-trailing-whitespace . t)))
 (sh-mode .  ((indent-tabs-mode . nil)
              (sh-indentation   . 2)
              (sh-basic-offset  . 2))))
