EXPORTS
    Init_gdk3
    rbgdk_rval2gdkpoints
    rbgdk_rval2gdkatoms
    rbgdk_rval2gdkpixbufglist
    gdk_atom_get_type
    gdk_geometry_get_type
    gdk_windowattr_get_type
    get_gdkatom
    rbgdk_gdkevent2rval
    rbgdk_rval2gdkevent

