#! /usr/bin/env ruby
=begin
  gtk2.rb

  Copyright (c) 2006 Ruby-GNOME2 Project Team
  This program is licenced under the same licence as Ruby-GNOME2.

  $Id: gtk2.rb,v 1.9 2006/06/17 13:18:12 mutoh Exp $
=end

require 'gtk2/base'

Gtk.init
